
#include "comb/composition-nz.h"

#include "comb/comb-print.h"
#include "bits/print-bin.h"
#include "nextarg.h"

//#include "aux0/sign.h"  // Abs() (with OEIS counting)
//#include "sort/convex.h"  // is_convex() (with OEIS counting)

#include "jjassert.h"

#include "fxttypes.h"
#include "fxtio.h"


//% Compositions of n into nonzero parts.

//#define TIMING  // uncomment to disable printing

//#define OEIS

int
main(int argc, char **argv)
{
    ulong n = 6;
    NXARG(n, "Compositions of n (n>=1)");

    composition_nz P(n);
    P.first();

    ulong ct = 0;

#ifdef TIMING
    do { ++ct; }  while  ( P.next() <= n );
#else

#ifdef OEIS
    ulong * st = new ulong[n+1];  // statistics
    for (ulong j=0; j<n; ++j)  st[j] = 0;
#endif  // OEIS

    const ulong *x = P.data();
    ulong k = 1;  // number of parts
    do
    {
#ifdef OEIS
        // OEIS sequences:
//        {  bool qq = true;
//            for (ulong j=0; j<k; ++j)  // A189101
//            {
//                if ( x[j]==5 )  { qq=false; break; }
//                if ( x[j]>7 )  { qq=false; break; }
//            }
//            if ( !qq )  continue;
//        }

//        {  bool qq = true;
//            if ( x[0]!=1 || x[k-1]!=1 )  qq = false;
//            else
//            {
//                for (ulong j=1; j<k; ++j)
//                    if ( Abs<long>((long)(x[j]-x[j-1])) > 1 ) { qq=false; break; }
//            }
//            if ( !qq )  continue;  // (sand piles by height)
//
//            ulong h = 0;  // A187081: height
//            for (ulong j=0; j<k; ++j)  if ( h<x[j] )  h=x[j];  // find max in x[]
//            st[h] += 1;
//            cout << h << "  ";
//        }

//        {  bool qq = true;
//            if ( x[0]!=1 )  qq = false;
//            else
//            {
//                for (ulong j=1; j<k; ++j)
//                    if ( (long)(x[j]-x[j-1]) > 1 ) { qq=false; break; }
//            }
//            if ( !qq )  continue;  // A005169 (fountains of n coins)
//
//            ulong h = 0;  // height: A187080
//            for (ulong j=0; j<k; ++j)  if ( h<x[j] )  h=x[j];  // find max in x[]
//            st[h] += 1;
//            cout << h << "  ";
//
////            st[k] += 1;  // length of base: A047998
//        }

//        { long t = test_convex(x, k);
//            if ( t < 0 )  continue;  // A000000 weakly convex (non-concave)
            // 1, 2, 3, 6, 10, 22, 43, 93, 195, 414, 863, 1802, 3712, 7616, 15521, 31505, 63689, 128425, 258355, 518945
//            if ( t > 0 )  continue;  // A000000 weakly concave (non-convex)
            // 0, 0, 1, 2, 6, 12, 30, 67, 153, 341, 748, 1607, 3411, 7140, 14791, 30390, 62027, 125947, 254734, 513669
//        }

        {
//        bool wxq = is_weakly_convex(x, k);
//        if ( !wxq )  continue;  // A000000 weakly convex
        // 1, 2, 4, 8, 15, 27, 47, 79, 130, 209, 330, 512, 784, 1183, 1765, 2604, 3804, 5504, 7898, 11240
//        if ( wxq ) cout << " wxq:" << endl;

//        bool waq = is_weakly_concave(x, k);
//        if ( !waq )  continue;  // A000000 weakly concave
        // 1, 2, 4, 7, 13, 21, 36, 57, 91, 140, 217, 323, 485, 711, 1039, 1494, 2144, 3032, 4279, 5970
//        if ( waq ) cout << " waq:" << endl;
        }

//        if ( test_strictly_convex(x, k) == 0 )  continue;  // A000000 strictly convex
        // 0, 0, 0, 1, 1, 3, 6, 10, 15, 24, 36, 53, 76, 105, 146, 200, 269, 360, 477, 624

//        if ( test_strictly_concave(x, k) == 0 )  continue;  // A000000 strictly concave
        // 0, 0, 0, 0, 1, 2, 3, 7, 11, 17, 25, 39, 55, 78, 108, 147, 201, 269, 355, 467

//        { bool xq, aq;
//            xq =  (test_strictly_convex(x, k) == 0 );
//            aq =  (test_strictly_concave(x, k) == 0 );
//            if ( (xq) && (aq) )  continue;  // A000000 neither
//            // 0, 0, 0, 1, 2, 5, 9, 17, 26, 41, 61, 92, 131, 183, 254, 347, 470, 629, 832, 1091
//            // first three terms are wrong here...
//        }

//        bool wq = is_weakly_convex(x, k),  sq = (0!=test_strictly_convex(x, k));
//        bool wq = is_weakly_concave(x, k),  sq = (0!=test_strictly_concave(x, k));
//        cout << "  wq=" << wq << "  sq=" << sq << "  " << (sq && !wq) << endl;
//        jjassert( !(sq && !wq) );

#endif  // OEIS

        ++ct;
        cout << setw(4) << ct << ":";
        print_vec("    ", x, k, true);
//        cout << "  " << k;
        cout << endl;
    }
    while ( (k=P.next()) <= n );
#endif


#ifdef OEIS
    cout << "  stats:  ";
    for (ulong j=0; j<=n; ++j)  cout << st[j] << " ";
    cout << endl;
    delete [] st;
#endif  // OEIS

    cout << "  ct=" << ct << endl;

    return 0;
}
// -------------------------


/*
Timing: (AMD Phenom II X4 945 3000MHz)

 time ./bin 31
arg 1: 31 == n  [Compositions of n (n>=1)]  default=6
  ct=1073741824
./bin 31  4.77s user 0.00s system 99% cpu 4.771 total
 ==> 1073741824/4.77 == 225,103,107 per second

*/

/*
Sand piles by height:
 for n in $(seq 1 20); do ./bin $n | g stats ; done | sed 's/stats://g;'
    0 1
    0 1 0
    0 1 0 0
    0 1 1 0 0
    0 1 2 0 0 0
    0 1 4 0 0 0 0
    0 1 7 0 0 0 0 0
    0 1 12 0 0 0 0 0 0
    0 1 20 1 0 0 0 0 0 0
    0 1 33 2 0 0 0 0 0 0 0
    0 1 54 5 0 0 0 0 0 0 0 0
    0 1 88 11 0 0 0 0 0 0 0 0 0
    0 1 143 22 0 0 0 0 0 0 0 0 0 0
    0 1 232 44 0 0 0 0 0 0 0 0 0 0 0
    0 1 376 84 0 0 0 0 0 0 0 0 0 0 0 0
    0 1 609 158 1 0 0 0 0 0 0 0 0 0 0 0 0
    0 1 986 293 2 0 0 0 0 0 0 0 0 0 0 0 0 0
    0 1 1596 535 5 0 0 0 0 0 0 0 0 0 0 0 0 0 0
    0 1 2583 969 12 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
    0 1 4180 1739 25 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0

column 2: Fibonacci - 1

 */


/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/composition-nz-demo.cc"
/// make-target2: "1demo DSRC=demo/comb/composition-nz-demo.cc DEMOFLAGS=-DTIMING"
/// End:
